package com.starcharge.wios.controller;

import com.alibaba.fastjson.JSONObject;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.HttpClientUtils;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.DeliveryConvert;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.Tauxiliarymaterials;
import com.starcharge.wios.dao.entity.WallboxApply;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dao.mappers.TauxiliarymaterialsMapper;
import com.starcharge.wios.dao.mappers.WallboxApplyMapper;
import com.starcharge.wios.dto.DeliveryUpdateDTO;
import com.starcharge.wios.enums2.LogisticStatus;
import com.starcharge.wios.enums2.ProductType;
import com.starcharge.wios.enums2.TauxiliarymaterialStatusEnum;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.DeliveryAddVo;
import com.starcharge.wios.vo.DeliveryExcelVo;
import io.netty.util.internal.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/818:10
 */
@RestController
@RequestMapping("api/delivery")
@Api(tags = "发货管理接口")
@Slf4j
public class DeliveryController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private DeliveryMapper deliveryMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private TauxiliarymaterialsMapper tauxiliarymaterialsMapper;
    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;
    @Value("${warehouseUrl.detailUrl}")
    private String detailUrl;
    @Autowired
    private OrderDao orderDao;
    @PostMapping("/add")
    @ApiOperation(value = "新增发货",notes = "新增发货，返回成功条数")
    @ParamsValidate
    @Transactional
    public BaseResponse<Integer> AddDelivery(@RequestBody @Validated(CreateEntityGroup.class) List<DeliveryAddVo> deliveryAddVoList, BindingResult bindingResult){
        //String orderid= snowFlakeService.getNextId(0,0)+"";
        return new BaseResponse(deliveryAddVoList.stream()
                .map(da->{
                    //SN号不能重复
                    if (!StringUtils.isEmpty(da.getMaterialSn())){
                        Delivery delivery=this.deliveryMapper.selectByMaterialSn(da.getMaterialSn());
                        if (!StringUtils.isEmpty(delivery)){
                            throw new ServiceException("当前SN号已经录入使用");
                        }
                    }
                    da.setInstallStatus(0);
                    Delivery delivery= DeliveryConvert.INSTANCE.convertAddVo(da);
                    delivery.setDeliveryOrder("FH"+NumberUtil.getRandomOrderId());
                    if(!StringUtils.isEmpty(delivery.getLogisticOrder())){
                        delivery.setLogisticStatus(TauxiliarymaterialStatusEnum.已发货.name());
                    }

                    //获取发货仓库信息
                    Map<String, String> paramSend = new HashMap<String, String>();
                    paramSend.put("id",da.getWarehouseId());
                    String jsonSend = HttpClientUtils.post(detailUrl,paramSend,"UTF-8", "UTF-8");
                    JSONObject jsonObjectSend=JSONObject.parseObject(jsonSend);
                    String data=jsonObjectSend.getString("data");
                    JSONObject jsonObjectData=JSONObject.parseObject(data);
                    delivery.setSender(jsonObjectData.getString("receivePeople"));
                    delivery.setSenderPhone(jsonObjectData.getString("receivePhone"));
                    delivery.setWarehouseId(da.getWarehouseId());
                    delivery.setWarehouseName(jsonObjectData.getString("name"));
                    if(!ProductType.辅材.name().equals(delivery.getIsproduct())){
                        //如果是墙盒 根据订单号去订单表里把墙盒名称 墙盒类别 厂商拿到并添加到发货表里
                        Order selectOrder = orderDao.selectById(da.getInstallOrder());
                        if (!StringUtils.isEmpty(selectOrder)){
                            //物料类型
                            delivery.setMaterialCategory(da.getIsproduct());
                            //厂商
                            if (!StringUtils.isEmpty(selectOrder.getOemName())){
                                delivery.setMaterialCompany(selectOrder.getOemName());
                            }
                            //物料编码
                            delivery.setMaterialCode(selectOrder.getItemModel());
                        }
                        //收货仓库的信息
                        WallboxApply wallboxApply=wallboxApplyMapper.selectWallboxApplyByOrderId(da.getInstallOrder());
                        if (!StringUtils.isEmpty(wallboxApply)){
                            delivery.setReceiver(wallboxApply.getReceiver());
                            delivery.setReceiverPhone(wallboxApply.getReceiverPhone());
                            delivery.setArrivalWarehouseId(wallboxApply.getReceiverWarehouseId());
                            delivery.setArrivalWarehouse(wallboxApply.getReceiverWarehouseName());

                            Map<String, String> paramArrival = new HashMap<String, String>();
                            paramArrival.put("id",wallboxApply.getReceiverWarehouseId());
                            paramArrival.put("materialCode",selectOrder.getItemModel());
                            String jsonArrival = HttpClientUtils.post(detailUrl,paramArrival,"UTF-8", "UTF-8");
                            JSONObject jsonObjectArrival=JSONObject.parseObject(jsonArrival);
                            String dataArrival=jsonObjectArrival.getString("data");
                            JSONObject jsonObjectDataArrival=JSONObject.parseObject(dataArrival);
                            //物料名称
                            if (!StringUtils.isEmpty(jsonObjectDataArrival.getString("materialsName"))){
                                delivery.setMaterialName(jsonObjectDataArrival.getString("materialsName"));
                            }
                            if (wallboxApply.getReceiverWarehouseId().equals(da.getWarehouseId())){
                                throw new ServiceException("400","发货仓库和到货仓库请选择不同的仓库!");
                            }
                        }
                        //申请团队
                        delivery.setTeam(wallboxApply.getSupplierTeam());
                        Delivery deliveryInstallOrder=deliveryMapper.selectByInstallOrder(da.getInstallOrder());
                        if (!StringUtils.isEmpty(deliveryInstallOrder)){
                            throw new ServiceException("400","订单号重复!");
                        }
                        int result=this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("1",delivery.getInstallOrder());
                        if(result!=1){
                            throw new ServiceException("400","更新订单发货状态失败！订单号："+delivery.getInstallOrder());
                        }
                    }else{
                        Tauxiliarymaterials tauxiliarymaterials = tauxiliarymaterialsMapper.selectByApplyOrder(da.getInstallOrder());
                        List<String> list=new ArrayList<>();
                        Integer quantity=0;
                        for (DeliveryAddVo deliveryAddVo:deliveryAddVoList){
                            quantity=quantity+deliveryAddVo.getQuantity();
                            if (list.contains(deliveryAddVo.getLogisticOrder())){
                                throw new ServiceException("400","相同申请单号下的多个物流单号不能重复!");
                            }else {
                                list.add(deliveryAddVo.getLogisticOrder());
                            }
                        }
                        if (!StringUtils.isEmpty(tauxiliarymaterials) && !StringUtils.isEmpty(tauxiliarymaterials.getApprovedQuantity())){
                            if (quantity.compareTo(Integer.parseInt(tauxiliarymaterials.getApprovedQuantity())) > 0){
                                throw new ServiceException("400","发货数量的总和不能大于批准数量!");
                            }
                        }
                        if (tauxiliarymaterials.getWarehouseId().equals(da.getWarehouseId())){
                            throw new ServiceException("400","发货仓库和到货仓库请选择不同的仓库!");
                        }
                        //收货仓库的信息
                        if (!StringUtils.isEmpty(tauxiliarymaterials)){
                            delivery.setReceiver(tauxiliarymaterials.getRecipents());
                            delivery.setReceiverPhone(tauxiliarymaterials.getReceivePhone());
                            delivery.setReceiveAddress(tauxiliarymaterials.getReceiveAddress());
                            delivery.setArrivalWarehouseId(da.getArrivalWarehouseId());
                            delivery.setArrivalWarehouse(tauxiliarymaterials.getWarehouseName());
                        }
                        delivery.setTeam(tauxiliarymaterials.getTeam());
                        delivery.setMaterialCode(tauxiliarymaterials.getMaterialCode());
                        delivery.setMaterialName(tauxiliarymaterials.getMaterialName());
                        delivery.setMaterialCategory(da.getIsproduct());

                        int result=this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.已发货.name(),delivery.getInstallOrder());
                        if(result!=1){
                            throw new ServiceException("400","更新辅材订单发货状态失败！订单号："+delivery.getInstallOrder());
                        }
                    }
                    this.commonUpdateService.UpdateColumns(delivery,true);
                    return this.deliveryMapper.insertSelective(delivery);
                }).reduce(0,Integer::sum));
    }
    @PostMapping("/query")
    @ApiOperation(value = "发货列表查询")
    public BaseResponse<List<Delivery>> queryDelivery(Delivery delivery){
        List<String> list= Arrays.asList(delivery.getLogisticStatus());
        delivery.setLogisticStatusList(list);
        return new BaseResponse(this.deliveryMapper.selectDeliveryList(delivery));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "发货详情")
    public BaseResponse<Delivery> getdetail(@RequestParam int id){
        return new BaseResponse(this.deliveryMapper.selectByPrimaryKey(id));
    }
    @GetMapping("/delete")
    @ApiOperation(value = "删除发货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.deliveryMapper.deleteByPrimaryKey(id));
    }

    @PostMapping("/update")
    @ApiOperation("更新发货单信息")
    @ParamsValidate
    @Transactional
    public BaseResponse<DeliveryUpdateDTO> update(@RequestBody @Validated(UpdateEntityGroup.class) DeliveryUpdateDTO deliveryUpdateDTO, BindingResult bindingResult){
        Delivery before=this.deliveryMapper.selectByPrimaryKey(deliveryUpdateDTO.getId());
        if(StringUtils.hasLength(before.getInstallOrder())&&!before.getInstallOrder().equals(deliveryUpdateDTO.getInstallOrder())){
            if(!ProductType.辅材.name().equals(deliveryUpdateDTO)){
                this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("0",before.getInstallOrder());
                this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("1",deliveryUpdateDTO.getInstallOrder());
            }else{
                this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.未发货.name(),before.getInstallOrder());
                this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.已发货.name(),deliveryUpdateDTO.getInstallOrder());
            }
        }
        if(StringUtils.isEmpty(deliveryUpdateDTO.getLogisticOrder())){
            deliveryUpdateDTO.setLogisticStatus(TauxiliarymaterialStatusEnum.未发货.name());
        }else{
            deliveryUpdateDTO.setLogisticStatus(TauxiliarymaterialStatusEnum.已发货.name());
        }
        this.commonUpdateService.UpdateColumns(deliveryUpdateDTO,false);
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO));
    }

    @PostMapping("/export")
    @ApiOperation(value = "导出发货单列表")
    public void export(HttpServletResponse response, @RequestBody Delivery Delivery) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<DeliveryExcelVo> data = DeliveryConvert.INSTANCE.convertExcelVoList(this
                .deliveryMapper.selectDeliveryList(Delivery));
        // 输出
        ExcelUtils.write(response, "发货管理列表.xls", "发货信息", DeliveryExcelVo.class, data);
    }

    @GetMapping("/supplierIdByOrderId")
    @ApiOperation(value = "根据安装单号获取服务商")
    public BaseResponse<WallboxApply> supplierIdByOrderId(@RequestParam String orderId){
        return new BaseResponse(this.wallboxApplyMapper.selectWallboxApplyByOrderId(orderId));
    }

}
